uniform sampler2D DiffuseMap;
uniform sampler2D NormalMap;
uniform sampler2D ParamMap;
uniform sampler2D NoiseMap;

uniform samplerCube BlurEnv;
uniform samplerCube ReflEnv;

uniform int 	ShaderType;
uniform vec4 	WindowColor;
uniform float paintSelected;
uniform float paintDead;

varying vec2 vTexCoord;
varying vec3 vNormal;
varying vec3 vTangent;
varying vec3 vBinormal;

varying vec4 vPosition;
varying vec4 vSpatial;

void main(void)
{
	vec3 lightDir = normalize(gl_LightSource[0].spotDirection);
	vec3 lightCol = gl_LightSource[0].diffuse.rgb;
	vec3 viewDir = normalize(vec4(0.0, 0.0, 0.0, 1.0) - vPosition);
	
	vec3 finalNormal = normalize(vNormal);
	
	if (ShaderType == 1 ||
		  ShaderType == 2)
	{
		finalNormal = faceforward(finalNormal, -viewDir, finalNormal);
	}
		
	vec3 finalTangent = normalize(vTangent);
	vec3 finalBinormal = normalize(vBinormal);
	
	vec4 normalSample = texture2D(NormalMap, vTexCoord);
	
	finalNormal += vTangent * (normalSample.g * 2.0 - 1.0);
	finalNormal += vBinormal * (normalSample.a * 2.0 - 1.0);
	
	finalNormal = normalize(finalNormal);

	float incidence = max(dot(viewDir, finalNormal), 0.0);
	incidence = 1.0 - (incidence * incidence);

	float kD = max(dot(finalNormal, lightDir), 0.0);
	
	vec3 half = normalize(lightDir + viewDir);
	
	float kS = max(dot(finalNormal, half), 0.0);
	kS = pow(kS, 40.0) * normalSample.b;
	
	vec4 diffuseSample = texture2D(DiffuseMap, vTexCoord);
	vec4 paramSample = texture2D(ParamMap, vTexCoord);
	
  vec3 reflvec = mat3(gl_ModelViewMatrixInverse) * reflect(viewDir, finalNormal);	

	vec3 blurryColor;
	blurryColor = diffuseSample.rgb * kD * lightCol * (1.0 + incidence);
	blurryColor += kS * normalSample.bbb * lightCol * (1.0 + incidence);
	blurryColor += diffuseSample.rgb * textureCube(BlurEnv, reflvec).rgb * normalSample.bbb * (1.0 + incidence);
	blurryColor += diffuseSample.rgb * textureCube(BlurEnv, finalNormal).rgb;

	vec3 specColor;
	specColor = diffuseSample.rgb * kD * lightCol + kS * lightCol * 2.0;
	specColor += diffuseSample.rgb * textureCube(ReflEnv, reflvec).rgb * normalSample.bbb * (1.0 + incidence);

  gl_FragColor.rgb = blurryColor.rgb * (1.0 - paramSample.a) + specColor.rgb * paramSample.aaa;
  gl_FragColor.rgb += gl_LightSource[1].diffuse.rgb * diffuseSample.rgb * incidence;
  
  if (ShaderType == 0) // standard solid
  {
	  gl_FragColor.rgb += diffuseSample.aaa * WindowColor.rgb;
	}
	else if (ShaderType == 1) // for the little windows on fighters
  {
  	gl_FragColor.a = diffuseSample.a;
  	gl_FragColor.rgb *= WindowColor.rgb;
  	gl_FragColor.rgb += WindowColor.rgb * normalSample.b * 0.33f;
  	gl_FragColor.rgb += kS;
  }
  else if (ShaderType == 2) // sprite
	{
		gl_FragColor.a = diffuseSample.a;
	}
	
	if (paintSelected > 0.0f)
	{
		gl_FragColor.rgb += vec3(2.0, 0.0, 0.0) * smoothstep(0.5, 1.0, incidence) * paintSelected;
	}
	else if (paintSelected < 0.0f)
	{
		gl_FragColor.rgb += vec3(0.0, 1.0, 2.0) * smoothstep(0.5, 1.0, incidence) * -paintSelected;
	}

	if (paintDead > 0.0f)
	{
		// mix 3 cadences of noise
		float noiseSample = texture2D(NoiseMap, vTexCoord).r * 0.5 + (texture2D(NoiseMap, vTexCoord * 3.3).r * 0.5 + texture2D(NoiseMap, vTexCoord * 10.89).r * 0.5) * 0.5;		
		float noiseClamp = clamp(noiseSample + 1.0 - 2.0 * paintDead, 0.0, 1.0);
		
		// blend between the normal surface texture, a thin burning area, and a "burnt" version based on the noise
		float ok = smoothstep(0.48, 0.52, noiseClamp);
		float burnt = 1.0 - ok;
		float burning = sqrt(ok * burnt);
		float explo = 1.0 - smoothstep(0.0, 0.02, noiseClamp);
		
		gl_FragColor.rgb = ok * gl_FragColor.rgb + burnt * kD * diffuseSample.rgb * vec3(0.5,0.3,0.1) * lightCol;
		gl_FragColor.rgb += burning * vec3(15.0, 6.0, 2.5) * noiseSample;
		gl_FragColor.rgb += explo * vec3(2.0, 4.0, 10.0);
	}
}
