uniform sampler2D DiffuseMap;
uniform sampler2D NormalMap;
uniform sampler2D ParamMap;

uniform samplerCube BlurEnv;
uniform samplerCube ReflEnv;

varying vec2 vTexCoord;
varying vec3 vNormal;
varying vec3 vTangent;
varying vec3 vBinormal;

varying vec4 vPosition;

void main(void)
{
	vec3 lightDir = normalize(vec3(0.0, 1.0, 1.0));
	vec3 viewDir = normalize(gl_ProjectionMatrixInverse * vec4(0.0, 0.0, 0.0, 1.0) - vPosition);
	
	vec3 finalTangent = normalize(vTangent);
	vec3 finalBinormal = normalize(vBinormal);
	
	vec3 finalNormal = normalize(vNormal);	
	
	vec4 normalSample = texture2D(NormalMap, vTexCoord);
	
	finalNormal += vTangent * (normalSample.g * 2.0 - 1.0);
	finalNormal += vBinormal * (normalSample.a * 2.0 - 1.0);
	
	finalNormal = normalize(finalNormal);

	float incidence = dot(viewDir, finalNormal);

	float kD = clamp(dot(finalNormal, lightDir), 0.0, 1.0);
	
	vec3 half = normalize(lightDir + viewDir);
	
	float kS = clamp(dot(finalNormal, half), 0.0, 1.0);
	kS = pow(kS, 80.0) * normalSample.b;
	
	vec4 diffuseSample = texture2D(DiffuseMap, vTexCoord);
	vec4 paramSample = texture2D(ParamMap, vTexCoord);
	
  vec3 reflvec = reflect(viewDir, finalNormal);	

	vec3 blurryColor;
	blurryColor = diffuseSample.rgb * (kD + kS) * (1.0 + (1.0 - incidence));
	blurryColor += textureCube(BlurEnv, reflvec).rgb * normalSample.bbb * (1.0 - incidence / 2.0);
		
	vec3 specColor;
	specColor = diffuseSample.rgb * kD + vec3(2.0,1.9, 1.8) * kS;
	specColor += textureCube(ReflEnv, reflvec).rgb * normalSample.bbb * (1.0 - incidence / 2.0);

  gl_FragColor.rgb = blurryColor.rgb * (1.0 - paramSample.a) + specColor.rgb * paramSample.aaa;
	gl_FragColor.rgb += diffuseSample.aaa * vec3(3.0, 2.4, 1.8);
}
